@@ -26,6 +26,9 @@
 #include "avilib.h"
 #include "sound.h"
 #include "v4l2uvc.h"
+#include "string_utils.h"
+#include "vcodecs.h"
+#include "acodecs.h"
 
 #define __AMUTEX &pdata->mutex
 #define __VMUTEX &videoIn->mutex
@@ -33,83 +36,92 @@
 #define __FMUTEX &global->file_mutex
 #define __GCOND  &global->IO_cond
 
-int initGlobals (struct GLOBAL *global) 
+int initGlobals (struct GLOBAL *global)
 {
 	__INIT_MUTEX( __GMUTEX );
 	__INIT_MUTEX( __FMUTEX );
 	__INIT_COND( __GCOND );   /* Initialized video buffer semaphore */
 
 	global->debug = DEBUG;
-	
+
 	const gchar *home = g_get_home_dir();
 
 	global->videodevice = g_strdup("/dev/video0");
-	
-	global->confPath = g_strjoin("/", home, ".guvcviewrc", NULL);
-	
+
+	global->confPath = g_strjoin("/", home, ".config", "guvcview", NULL);
+	int ret = g_mkdir_with_parents(global->confPath, 0777);
+	if(ret)
+		fprintf(stderr, "Couldn't create configuration dir: %s \n", global->confPath);
+
+	g_free(global->confPath);
+	global->confPath = g_strjoin("/", home, ".config", "guvcview", "video0", NULL);
+
 	global->vidFPath = g_new(pchar, 2);
-	
+
 	global->imgFPath = g_new(pchar, 2);
-	
+
 	global->profile_FPath = g_new(pchar, 2);
-	
+
 	global->vidFPath[1] = g_strdup(home);
-	
+
 	global->imgFPath[1] = g_strdup(home);
-	
+
 	global->profile_FPath[1] = g_strdup(home);
-	
-	global->vidFPath[0] = g_strdup(DEFAULT_AVI_FNAME);
-	
-	global->imgFPath[0] = g_strdup(DEFAULT_IMAGE_FNAME);
-	
+
+	global->vidFPath[0] = g_strdup("guvcview_video.mkv");
+	global->VidFormat = check_video_type(global->vidFPath[0]);
+
+	global->imgFPath[0] = g_strdup("guvcview_image.jpg");
+	global->imgFormat = check_image_type(global->imgFPath[0]);
+
 	global->profile_FPath[0] = g_strdup("default.gpfl");
-	
+
 	global->WVcaption = g_new(char, 32);
-	
+
 	g_snprintf(global->WVcaption,10,"GUVCVIdeo");
 
 	global->videoBuff = NULL;
 	global->video_buff_size = VIDBUFF_SIZE;
-	
-	global->stack_size=TSTACK;
-	
-	global->image_inc=0;
 
-	global->imageinc_str = g_new(char, 25);
-	g_snprintf(global->imageinc_str,20,_("File num:%d"),global->image_inc);
-	
-	global->vidinc_str = g_new(char, 25);
-	g_snprintf(global->vidinc_str,20,_("File num:%d"),global->vid_inc);
-	
+	global->image_inc = 1; //increment filename by default
+	global->vid_inc = 1;   //increment filename by default
+
 	global->vid_sleep=0;
 	global->vidfile=NULL; /*vid filename passed through argument options with -n */
 	global->Capture_time=0; /*vid capture time passed through argument options with -t */
 	global->lprofile=0; /* flag for -l command line option*/
-	global->imgFormat=0; /* 0 -JPG 1-BMP 2-PNG*/
-	global->VidCodec=0; /*0-"MJPG"  1-"YUY2" 2-"DIB "(rgb32) 3-...*/ 
-	global->AVI_MAX_LEN=AVI_MAX_SIZE; /* ~2 Gb*/    
+
+	/** try to set video codec default to mpeg4*/
+	setVcodecVal ();
+	int vcodec = get_list_vcodec_index(AV_CODEC_ID_MPEG4);
+	if(vcodec < 0)
+		vcodec = 0;
+	global->VidCodec= vcodec; /*0-"MJPG"  1-"YUY2" 2-"DIB "(rgb32) 3-...*/
+	global->VidCodec_ID = get_vcodec_id(global->VidCodec); // AV_CODEC_ID_MPEG4 or MJPG
+	/** try to set audio codec default to mp2*/
+	setAcodecVal();
+	int acodec = get_list_acodec_index(AV_CODEC_ID_MP2);
+	if(acodec < 0)
+		acodec = 0;
+	global->AudCodec = acodec; /*0-"PCM"  1-"MPG2" 2-...*/
+
 	global->av_drift=0;
-	global->currtime=0;
-	global->lasttime=0;
 	global->Vidstarttime=0;
 	global->Vidstoptime=0;
 	global->framecount=0;
 	global->w_ind=0;
 	global->r_ind=0;
-		
+
 	global->Sound_enable=TRUE; /*Enable Sound by Default*/
 	global->Sound_IndexDev=NULL;
 	global->Sound_SampRate=SAMPLE_RATE;
 	global->Sound_SampRateInd=0;
 	global->Sound_numInputDev=0;
 	global->Sound_Format=WAVE_FORMAT_MPEG12; /*set MP2 by default*/
-	global->Sound_DefDev=0; 
+	global->Sound_DefDev=0;
 	global->Sound_UseDev=0;
 	global->Sound_NumChan=NUM_CHANNELS;
 	global->Sound_NumChanInd=0;
-	global->Sound_delay=0;
-	//global->Sound_bitRate=160; /*160 Kbps = 20000 Bps*/
 	global->Sound_delay=0;     /*sound delay in nanosec*/
 	global->FpsCount=0;
 
@@ -135,33 +147,38 @@ int initGlobals (struct GLOBAL *global)
 	global->height = DEFAULT_HEIGHT;
 	global->winwidth=WINSIZEX;
 	global->winheight=WINSIZEY;
-	global->spinbehave=0;
+
 	global->default_action=0;
-	global->boxvsize=0;
-	
+
 	global->mode = g_new(char, 6);
 	g_snprintf(global->mode, 5, "mjpg");
-	
+
 	global->format = V4L2_PIX_FMT_MJPEG;
 
 	global->Frame_Flags = YUV_NOFILT;
 	global->osdFlags = 0;
-	
+
     global->no_display = FALSE;
 	global->exit_on_close = FALSE;
 	global->skip_n=0;
 	global->jpeg=NULL;
+	global->uvc_h264_unit = 0; //not supported by default
+
 	/* reset with videoIn parameters */
 	global->autofocus = FALSE;
 	global->AFcontrol = FALSE;
 	global->VidButtPress = FALSE;
 	global->change_res = FALSE;
 	global->add_ctrls = FALSE;
+	global->lctl_method = LIST_CTL_METHOD_NEXT_FLAG; //next_ctrl flag method
 	return (0);
 }
 
 int closeGlobals(struct GLOBAL *global)
 {
+	if(!global)
+		return(-1);
+
 	g_free(global->videodevice);
 	g_free(global->confPath);
 	g_free(global->vidFPath[1]);
@@ -174,15 +191,13 @@ int closeGlobals(struct GLOBAL *global)
 	g_free(global->imgFPath);
 	g_free(global->profile_FPath);
 	g_free (global->WVcaption);
-	g_free(global->imageinc_str);
-	g_free(global->vidinc_str);
 	g_free(global->vidfile);
 	g_free(global->mode);
 	g_free(global->Sound_IndexDev);
 	__CLOSE_MUTEX( __GMUTEX );
 	__CLOSE_MUTEX( __FMUTEX );
 	__CLOSE_COND( __GCOND );
-	
+
 	global->videodevice=NULL;
 	global->confPath=NULL;
 	global->vidfile=NULL;
@@ -190,6 +205,5 @@ int closeGlobals(struct GLOBAL *global)
 	if(global->jpeg) g_free(global->jpeg);
 	global->jpeg=NULL;
 	g_free(global);
-	global=NULL;
 	return (0);
 }
