@@ -27,7 +27,7 @@
 #include "defs.h"
 #include "utils.h"
 
-typedef struct _sndDev 
+typedef struct _sndDev
 {
 	int id;
 	int chan;
@@ -44,25 +44,26 @@ typedef struct _VidBuff
 	QWORD time_stamp;
 	BYTE *frame;
 	int bytes_used;
+	gboolean keyframe;
 } VidBuff;
 
 /*global variables used in guvcview*/
-struct GLOBAL 
+struct GLOBAL
 {
-	  	
+
 	__MUTEX_TYPE mutex;    //global struct mutex
 	__MUTEX_TYPE file_mutex; //video file mutex
 	__COND_TYPE  IO_cond;      //IO thread semaphore
 
 	VidBuff *videoBuff;    //video Buffer
-	int video_buff_size;   //size in frmaes of video buffer
-	
+	int video_buff_size;   //size in frames of video buffer
+
 	char *videodevice;     // video device (def. /dev/video0)
 	char *confPath;        //configuration file path
 	char *vidfile;         //video filename passed through argument options with -n
 	char *WVcaption;       //video preview title bar caption
-	char *imageinc_str;    //label for File inc
-	char *vidinc_str;      //label for File inc
+	//char *imageinc_str;    //label for File inc
+	//char *vidinc_str;      //label for File inc
 	char *mode;            //mjpg (default)
 	pchar* vidFPath;       //video path [0] - filename  [1] - dir
 	pchar* imgFPath;       //image path [0] - filename  [1] - dir
@@ -71,25 +72,23 @@ struct GLOBAL
 	sndDev *Sound_IndexDev;//list of sound input devices
 	BYTE *jpeg;            // jpeg buffer
 
-	ULONG AVI_MAX_LEN;     //avi max length
 	int64_t av_drift;      // amount of A/V time correction
-	DWORD currtime;
-	DWORD lasttime;
-	UINT64 Vidstarttime;    //video start time
-	UINT64 Vidstoptime;     //video stop time
+	UINT64 Vidstarttime;   //video start time
+	UINT64 Vidstoptime;    //video stop time
 	QWORD v_ts;            //video time stamp
 	QWORD a_ts;            //audio time stamp
-	DWORD vid_inc;         //video name increment
-	DWORD framecount;      //video frame count
+	uint64_t vid_inc;      //video name increment
+	uint64_t framecount;   //video frame count
 	DWORD frmCount;        //frame count for fps display calc
-	DWORD image_inc;       //image name increment
+	uint64_t image_inc;    //image name increment
 
-	int stack_size;        //thread stack size
 	int vid_sleep;         //video thread sleep time (0 by default)
 	int cap_meth;          //capture method: 1-mmap 2-read
-	int Capture_time;      //video capture time passed through argument options with -t 
+	int Capture_time;      //video capture time passed through argument options with -t
 	int imgFormat;         //image format: 0-"jpg", 1-"png", 2-"bmp"
 	int VidCodec;          //0-"MJPG"  1-"YUY2" 2-"DIB "(rgb32) 3-....
+	int VidCodec_ID;       //lavc codec ID
+	int AudCodec;          //0-PCM 1-MPG2 3-...
 	int VidFormat;         //0-AVI 1-MKV ....
 	int Sound_API;         //audio API: 0-PORTAUDIO 1-PULSEAUDIO
 	int Sound_SampRate;    //audio sample rate
@@ -100,8 +99,7 @@ struct GLOBAL
 	int Sound_NumChan;     //audio number of channels
 	int Sound_NumChanInd;  //audio number of channels combo index
 	WORD Sound_Format;     //audio codec - fourcc (avilib.h)
-	//int Sound_bitRate;     //bit rate for mpeg audio compression
-	UINT64 Sound_delay;    //sound delay in nanosec
+	uint64_t Sound_delay;  //added sound delay (in nanosec)
 	int PanStep;           //step angle for Pan
 	int TiltStep;          //step angle for Tilt
 	int FpsCount;          //frames counter for fps calc
@@ -122,8 +120,6 @@ struct GLOBAL
 	int height;            //frame height
 	int winwidth;          //control windoe width
 	int winheight;         //control window height
-	int boxvsize;          //size of vertical spliter
-	int spinbehave;        //spin: 0-non editable 1-editable
 	int format;            //v4l2 pixel format
 	int Frame_Flags;       //frame filter flags
 	int osdFlags;          // Flags to control onscreen display
@@ -131,6 +127,8 @@ struct GLOBAL
 	int w_ind;             //write frame index
 	int r_ind;             //read  frame index
 	int default_action;    // 0 for taking picture, 1 for video
+	int lctl_method;       // 0 for control id loop, 1 for next_ctrl flag method
+	int uvc_h264_unit;     //uvc h264 unit id, if <= 0 then uvc h264 is not supported
 
 	float DispFps;         //fps value
 
@@ -153,6 +151,7 @@ struct GLOBAL
 	gboolean control_only; //if set don't stream video (enables image control in other apps e.g. ekiga, skype, mplayer)
 	gboolean change_res;   //flag for reseting resolution
 	gboolean add_ctrls;    //flag for exiting after adding extension controls
+	gboolean monotonic_pts;//flag if we are using monotonic or real pts
 };
 
 /*----------------------------- prototypes ------------------------------------*/
