@@ -35,55 +35,105 @@
 /* Must set this as global so they */
 /* can be set from any callback.   */
 
-struct GWIDGET 
+struct GWIDGET
 {
+	/*the main loop : only needed for no_display option*/
+	GMainLoop *main_loop;
+
 	/* The main window*/
 	GtkWidget *mainwin;
 	/* A restart Dialog */
 	GtkWidget *restartdialog;
 	/*Paned containers*/
-	GtkWidget *boxv;
+	GtkWidget *maintable;
 	GtkWidget *boxh;
 
-	GtkWidget *VidCodec;
+	//group list for menu video codecs
+	GSList *vgroup;
+	//group list for menu audio codecs
+	GSList *agroup;
+
+	//menu top widgets
+	GtkWidget *menu_photo_top;
+	GtkWidget *menu_video_top;
+
+	GtkWidget *status_bar;
+
 	GtkWidget *label_SndAPI;
 	GtkWidget *SndAPI;
-	GtkWidget *SndEnable; 
+	GtkWidget *SndEnable;
 	GtkWidget *SndSampleRate;
 	GtkWidget *SndDevice;
 	GtkWidget *SndNumChan;
 	GtkWidget *SndComp;
 	/*must be called from main loop if capture timer enabled*/
-	GtkWidget *ImageFNameEntry;
-	GtkWidget *ImgFileButt;
-	GtkWidget *VidFileButt;
 	GtkWidget *ImageType;
 	GtkWidget *CapImageButt;
-	GtkWidget *ImageInc;
-	GtkWidget *ImageIncLabel;
 	GtkWidget *CapVidButt;
-	GtkWidget *VidButton_Img;
-	GtkWidget *VidFNameEntry;
-	GtkWidget *VidIncLabel;
-	GtkWidget *VidInc;
-	GtkWidget *VidFormat;
 	GtkWidget *Resolution;
 	GtkWidget *InpType;
 	GtkWidget *FrameRate;
 	GtkWidget *Devices;
-	GtkWidget *FileDialog;
-	GtkWidget *lavc_button;
-	GtkWidget *lavc_aud_button;
 	GtkWidget *jpeg_comp;
 	GtkWidget *quitButton;
-	
-	GtkWidget *TakeImageByDefault;
-	GtkWidget *TakeVidByDefault;
-	
+
 	gboolean vid_widget_state;
+	int status_warning_id;
 };
 
-struct ALL_DATA 
+/* uvc H264 control widgets */
+struct uvc_h264_gtkcontrols
+{
+	GtkWidget* FrameInterval;
+	GtkWidget* BitRate;
+	GtkWidget* Hints_res;
+	GtkWidget* Hints_prof;
+	GtkWidget* Hints_ratecontrol;
+	GtkWidget* Hints_usage;
+	GtkWidget* Hints_slicemode;
+	GtkWidget* Hints_sliceunit;
+	GtkWidget* Hints_view;
+	GtkWidget* Hints_temporal;
+	GtkWidget* Hints_snr;
+	GtkWidget* Hints_spatial;
+	GtkWidget* Hints_spatiallayer;
+	GtkWidget* Hints_frameinterval;
+	GtkWidget* Hints_leakybucket;
+	GtkWidget* Hints_bitrate;
+	GtkWidget* Hints_cabac;
+	GtkWidget* Hints_iframe;
+	GtkWidget* Resolution;
+	GtkWidget* SliceUnits;
+	GtkWidget* SliceMode;
+	GtkWidget* Profile;
+	GtkWidget* Profile_flags;
+	GtkWidget* IFramePeriod;
+	GtkWidget* EstimatedVideoDelay;
+	GtkWidget* EstimatedMaxConfigDelay;
+	GtkWidget* UsageType;
+	//GtkWidget* UCConfig;
+	GtkWidget* RateControlMode;
+	GtkWidget* RateControlMode_cbr_flag;
+	GtkWidget* TemporalScaleMode;
+	GtkWidget* SpatialScaleMode;
+	GtkWidget* SNRScaleMode;
+	GtkWidget* StreamMuxOption;
+	GtkWidget* StreamMuxOption_aux;
+	GtkWidget* StreamMuxOption_mjpgcontainer;
+	GtkWidget* StreamFormat;
+	GtkWidget* EntropyCABAC;
+	GtkWidget* Timestamp;
+	GtkWidget* NumOfReorderFrames;
+	GtkWidget* PreviewFlipped;
+	GtkWidget* View;
+	GtkWidget* StreamID;
+	GtkWidget* SpatialLayerRatio;
+	GtkWidget* LeakyBucketSize;
+	GtkWidget* probe_button;
+	GtkWidget* commit_button;
+};
+
+struct ALL_DATA
 {
 	struct paRecordData *pdata;
 	struct GLOBAL *global;
@@ -91,6 +141,7 @@ struct ALL_DATA
 	struct vdIn *videoIn;
 	struct VideoFormatData *videoF;
 	struct GWIDGET *gwidget;
+	struct uvc_h264_gtkcontrols  *h264_controls;
 	struct VidState *s;
 	__THREAD_TYPE video_thread;
 	__THREAD_TYPE audio_thread;
