@@ -1,5 +1,5 @@
 /*******************************************************************************#
-#           guvcview              http://guvcview.berlios.de                    #
+#           guvcview              http://guvcview.sourceforge.net               #
 #                                                                               #
 #           Paulo Assis <pj.assis@gmail.com>                                    #
 #           Nobuhiro Iwamatsu <iwamatsu@nigauri.org>                            #
@@ -53,38 +53,42 @@ draw_controls (struct ALL_DATA *all_data)
 	struct GLOBAL *global = all_data->global;
 	struct vdIn *videoIn = all_data->videoIn;
 	struct focusData *AFdata = all_data->AFdata;
-    
-    if (s->control_list) 
+
+    if (s->control_list)
     {
         free_control_list(s->control_list);
     }
     s->num_controls = 0;
     //get the control list
-    s->control_list = get_control_list(videoIn->fd, &(s->num_controls));
+    s->control_list = get_control_list(videoIn->fd, &(s->num_controls), global->lctl_method);
 
     if(!s->control_list)
     {
         printf("Error: empty control list\n");
         return;
     }
-    
+
     get_ctrl_values (videoIn->fd, s->control_list, s->num_controls, NULL);
-    
-    s->table = gtk_table_new (1, 3, FALSE);
-    gtk_table_set_row_spacings (GTK_TABLE (s->table), 4);
-    gtk_table_set_col_spacings (GTK_TABLE (s->table), 4);
-    gtk_container_set_border_width (GTK_CONTAINER (s->table), 2);
-    
+
+    s->table = gtk_grid_new ();
+   	gtk_grid_set_column_homogeneous (GTK_GRID(s->table), FALSE);
+	gtk_widget_set_hexpand (s->table, TRUE);
+	gtk_widget_set_halign (s->table, GTK_ALIGN_FILL);
+
+	gtk_grid_set_row_spacing (GTK_GRID(s->table), 4);
+	gtk_grid_set_column_spacing (GTK_GRID (s->table), 4);
+	gtk_container_set_border_width (GTK_CONTAINER (s->table), 2);
+
     //add control widgets to control list
     create_control_widgets(s->control_list, (void *) all_data, global->control_only, global->debug);
-    
+
     int done = 0;
     int row=0;
-    
+
     Control *current = s->control_list;
     Control *next = current->next;
 
-    while(!done) 
+    while(!done)
     {
         //add some flags
         if ((current->control.id == V4L2_CID_PAN_RELATIVE) ||
@@ -92,28 +96,28 @@ draw_controls (struct ALL_DATA *all_data)
 	    {
 	        videoIn->PanTilt++;
 	    }
-	    
+
         //special cases (extra software controls)
 	    if (((current->control.id == V4L2_CID_FOCUS_ABSOLUTE) ||
 	        (current->control.id == V4L2_CID_FOCUS_LOGITECH)) &&
-	         !(global->control_only)) 
+	         !(global->control_only))
 	    {
 		    global->AFcontrol=1;
-				
+
 			if(!AFdata)
 			{
-			    AFdata = initFocusData(current->control.maximum, 
-			        current->control.minimum, 
-			        current->control.step, 
+			    AFdata = initFocusData(current->control.maximum,
+			        current->control.minimum,
+			        current->control.step,
 			        current->control.id);
 				all_data->AFdata = AFdata;
 			}
-				
-			if(!AFdata) 
+
+			if(!AFdata)
 			    global->AFcontrol = 0;
 			else
 			{
-				GtkWidget *Focus_box = gtk_hbox_new (FALSE, 0);
+				GtkWidget *Focus_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 				GtkWidget *AutoFocus = gtk_check_button_new_with_label (_("Auto Focus (continuous)"));
 				GtkWidget *FocusButton = gtk_button_new_with_label (_("set Focus"));
 				gtk_box_pack_start (GTK_BOX (Focus_box), AutoFocus, TRUE, TRUE, 0);
@@ -121,34 +125,34 @@ draw_controls (struct ALL_DATA *all_data)
 				gtk_widget_show (Focus_box);
 				gtk_widget_show (AutoFocus);
 				gtk_widget_show (FocusButton);
-				gtk_table_attach (GTK_TABLE (s->table), Focus_box, 1, 2, row, row+1,
-				    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
-			
+				gtk_grid_attach(GTK_GRID(s->table), Focus_box, 1, row, 1, 1);
+
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (AutoFocus),
 				    global->autofocus ? TRUE: FALSE);
-				
+
 				g_signal_connect (G_OBJECT (AutoFocus), "toggled",
 				    G_CALLBACK (autofocus_changed), all_data);
 				g_signal_connect (G_OBJECT (FocusButton), "clicked",
 				    G_CALLBACK (setfocus_clicked), all_data);
-				
+
 				row++; /*increment control row*/
-				
+
 			}
 		}
- 
-        
+
+
         if(current->label)
-            gtk_table_attach (GTK_TABLE (s->table), current->label, 0, 1, row, row+1,
-                GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+            gtk_grid_attach(GTK_GRID(s->table), current->label, 0, row, 1 , 1);
         if(current->widget)
-            gtk_table_attach (GTK_TABLE (s->table), current->widget, 1, 2, row, row+1,
-                GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+        {
+            gtk_grid_attach(GTK_GRID(s->table), current->widget, 1, row, 1 , 1);
+            gtk_widget_set_halign (current->widget, GTK_ALIGN_FILL);
+			gtk_widget_set_hexpand (current->widget, TRUE);
+        }
         if(current->spinbutton)
-            gtk_table_attach (GTK_TABLE (s->table), current->spinbutton, 2, 3, row, row+1,
-                GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
-        
-                
+            gtk_grid_attach(GTK_GRID(s->table), current->spinbutton, 2, row, 1 , 1);
+
+
         if(next == NULL)
             done = 1;
         else
@@ -157,14 +161,14 @@ draw_controls (struct ALL_DATA *all_data)
             current = next;
             next = current->next;
         }
-    }	
+    }
 
 	/*              try to start the video stream             */
 	/* do it here (after all ioctls) since some cameras take  */
 	/* a long time to initialize after this                   */
 	/* it's OK if it fails since it is retried in uvcGrab     */
 	if(!global->control_only) video_enable(videoIn);
-	
+
 	s = NULL;
 	global = NULL;
 	videoIn = NULL;
