@@ -25,6 +25,14 @@
 #ifndef GUVC_MATROSKA_H
 #define GUVC_MATROSKA_H
 
+#include "defs.h"
+#include <inttypes.h>
+#include <sys/types.h>
+#include <glib.h>
+#include <pthread.h>
+#include "file_io.h"
+#include "io_stream.h"
+
 /* EBML version supported */
 #define EBML_VERSION 1
 
@@ -218,10 +226,134 @@ typedef enum {
   MATROSKA_TRACK_ENCODING_COMP_HEADERSTRIP = 3,
 } MatroskaTrackEncodingCompAlgo;
 */
+
+typedef struct ebml_master {
+    int64_t         pos;                ///< absolute offset in the file where the master's elements start
+    int             sizebytes;          ///< how many bytes were reserved for the size
+} ebml_master;
+
+typedef struct mkv_seekhead_entry {
+    unsigned int    elementid;
+    uint64_t        segmentpos;
+} mkv_seekhead_entry;
+
+typedef struct mkv_seekhead {
+    int64_t                 filepos;
+    int64_t                 segment_offset;     ///< the file offset to the beginning of the segment
+    int                     reserved_size;      ///< -1 if appending to file
+    int                     max_entries;
+    mkv_seekhead_entry      *entries;
+    int                     num_entries;
+} mkv_seekhead;
+
+typedef struct {
+    uint64_t        pts;
+    int             tracknum;
+    int64_t         cluster_pos;        ///< file offset of the cluster containing the block
+} mkv_cuepoint;
+
+typedef struct {
+    int64_t         segment_offset;
+    mkv_cuepoint    *entries;
+    int             num_entries;
+} mkv_cues;
+
+typedef struct mkv_Context {
+    int             mode; //matroska or webm
+    io_Writer*      writer;
+    ebml_master     segment;
+    int64_t         segment_offset;
+    ebml_master     cluster;
+    int64_t         cluster_pos;        ///< file offset of the current cluster
+    int64_t         cluster_pts;
+    int64_t         duration_offset;
+    int64_t         duration;
+    mkv_seekhead    *main_seekhead;
+    mkv_cues        *cues;
+
+	uint64_t      timescale;
+	uint64_t      first_pts; //pts of first packet
+	// store one audio packet (webm)
+    unsigned int  pkt_buffer_size;
+    BYTE*         pkt_buffer;
+    unsigned int  pkt_size;
+    uint64_t      pkt_pts;
+    int           pkt_duration;
+    int           pkt_stream_index;
+    int           pkt_flags;
+
+    io_Stream* stream_list;
+    int stream_list_size;
+} mkv_Context;
+
+
+/** create a muxer context
+ * mode : WEBM_FORMAT or MKV_FORMAT*/
+mkv_Context*
+mkv_create_context(char* filename, int mode);
+
+/** add a video stream to the context */
+io_Stream*
+mkv_add_video_stream(mkv_Context *MKV,
+					int32_t width,
+					int32_t height,
+					int32_t fps,
+					int32_t fps_num,
+					int32_t codec_id);
+
+/** add a audio stream to the context */
+io_Stream*
+mkv_add_audio_stream(mkv_Context *MKV,
+					int32_t   channels,
+					int32_t   rate,
+					int32_t   bits,
+					int32_t   mpgrate,
+					int32_t   codec_id,
+					int32_t   format);
+
+/** write the header*/
+int mkv_write_header(mkv_Context* MKV);
+
+int mkv_write_packet(mkv_Context* MKV,
+					int stream_index,
+					BYTE* data,
+                    int size,
+                    int duration,
+                    uint64_t pts,
+                    int flags);
+
+/** finalize file operations*/
+int mkv_close(mkv_Context* MKV);
+
+/** destroy the muxer context (clean up)*/
+void mkv_destroy_context(mkv_Context* MKV);
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 typedef struct mk_Writer mk_Writer;
 
-mk_Writer *mk_createWriter( const char *filename );
+mk_Writer *mk_createWriter( const char *filename, int format );
 
 int  mk_writeHeader( mk_Writer *w, const char *writingApp,
                      const char *codecID,
