@@ -39,8 +39,8 @@
 #include "v4l2uvc.h"
 #include "../config.h"
 /*----------------------- write conf (.guvcviewrc) file ----------------------*/
-int 
-writeConf(struct GLOBAL *global, char *videodevice) 
+int
+writeConf(struct GLOBAL *global, char *videodevice)
 {
 	int ret=0;
 	FILE *fp;
@@ -50,28 +50,22 @@ writeConf(struct GLOBAL *global, char *videodevice)
     vcodecs_data *vcodec_defaults = get_codec_defaults(global->VidCodec);
     acodecs_data *acodec_defaults = get_aud_codec_defaults(get_ind_by4cc(global->Sound_Format));
 	// write to tmp file then rename after sucessfull fsync
-	// using fsync avois data loss on system crash 
+	// using fsync avois data loss on system crash
 	// see: https://bugs.launchpad.net/ubuntu/+source/linux/+bug/317781/comments/54
 	char *tmpfile = g_strjoin (".", global->confPath, "tmp", NULL);
-	if ((fp = g_fopen(tmpfile,"w"))!=NULL) 
+	if ((fp = g_fopen(tmpfile,"w"))!=NULL)
 	{
 		g_fprintf(fp,"# guvcview configuration file for %s\n\n",videodevice);
 		g_fprintf(fp,"version='%s'\n",VERSION);
-		g_fprintf(fp,"# Thread stack size: default 128 pages of 64k = 8388608 bytes\n");
-		g_fprintf(fp,"stack_size=%d\n",global->stack_size);
 		g_fprintf(fp,"# video loop sleep time in ms: 0,1,2,3,...\n");
 		g_fprintf(fp,"# increased sleep time -> less cpu load, more droped frames\n");
 		g_fprintf(fp,"vid_sleep=%i\n",global->vid_sleep);
 		g_fprintf(fp,"# capture method: 1- mmap 2- read\n");
 		g_fprintf(fp,"cap_meth=%i\n",global->cap_meth);
 		g_fprintf(fp,"# video resolution \n");
-		g_fprintf(fp,"resolution='%ix%i'\n",global->width,global->height);
-		g_fprintf(fp,"# control window size: default %ix%i\n",WINSIZEX,WINSIZEY);
-		g_fprintf(fp,"windowsize='%ix%i'\n",global->winwidth,global->winheight);
-		g_fprintf(fp,"#vertical pane size\n");
-		g_fprintf(fp,"vpane=%i\n",global->boxvsize);
-		g_fprintf(fp,"#spin button behavior: 0-non editable 1-editable\n");
-		g_fprintf(fp,"spinbehave=%i\n", global->spinbehave);
+		g_fprintf(fp,"resolution='%ix%i'\n", global->width, global->height);
+		g_fprintf(fp,"# control window size: default %ix%i\n", WINSIZEX, WINSIZEY);
+		g_fprintf(fp,"windowsize='%ix%i'\n",global->winwidth, global->winheight);
 		g_fprintf(fp,"#Default action. 0 for picture, 1 for video.\n");
 		g_fprintf(fp,"default_action=%i\n", global->default_action);
 		g_fprintf(fp,"# mode video format 'yuvy' 'yvyu' 'uyvy' 'yyuv' 'yu12' 'yv12' 'nv12' 'nv21' 'nv16' 'nv61' 'y41p' 'grey' 'y10b' 'y16 ' 's501' 's505' 's508' 'gbrg' 'grbg' 'ba81' 'rggb' 'rgb3' 'bgr3' 'jpeg' 'mjpg'(default)\n");
@@ -88,35 +82,30 @@ writeConf(struct GLOBAL *global, char *videodevice)
 		g_fprintf(fp,"hwaccel=%i\n",global->hwaccel);
 		g_fprintf(fp,"# video compression format: 0-MJPG 1-YUY2/UYVY 2-DIB (BMP 24) 3-MPEG1 4-FLV1 5-MPEG2 6-MS MPEG4 V3(DIV3) 7-MPEG4 (DIV5)\n");
 		g_fprintf(fp,"vid_codec=%i\n",global->VidCodec);
-		g_fprintf(fp,"# avi file max size (MAX: %d bytes)\n",AVI_MAX_SIZE);
-		g_fprintf(fp,"avi_max_len=%li\n",global->AVI_MAX_LEN);
+		g_fprintf(fp,"# video muxer format: 0-AVI 1-MKV 2-WebM\n");
+		g_fprintf(fp,"vid_format=%i\n",global->VidFormat);
 		g_fprintf(fp,"# Auto Video naming (ex: filename-n.avi)\n");
-		g_fprintf(fp,"vid_inc=%d\n",global->vid_inc);
+		g_fprintf(fp,"vid_inc=%i\n",(global->vid_inc > 0 ? 1: 0));
 		g_fprintf(fp,"# sound 0 - disable 1 - enable\n");
 		g_fprintf(fp,"sound=%i\n",global->Sound_enable);
 		g_fprintf(fp,"# sound API: 0- Portaudio  1- Pulseaudio\n");
 		g_fprintf(fp,"snd_api=%i\n", global->Sound_API);
-		g_fprintf(fp,"# snd_device - sound device id as listed by portaudio (pulse uses default device)\n");
+		g_fprintf(fp,"# snd_device - sound device index/id as listed by audio API\n");
 		g_fprintf(fp,"snd_device=%i\n",global->Sound_UseDev);
 		g_fprintf(fp,"# snd_samprate - sound sample rate\n");
 		g_fprintf(fp,"snd_samprate=%i\n",global->Sound_SampRateInd);
 		g_fprintf(fp,"# snd_numchan - sound number of channels 0- dev def 1 - mono 2 -stereo\n");
 		g_fprintf(fp,"snd_numchan=%i\n",global->Sound_NumChanInd);
-		g_fprintf(fp,"#snd_numsec - video audio blocks size in sec: 1,2,3,.. \n");
 		g_fprintf(fp,"# sound delay in nanosec - delays sound by the specified amount when capturing video\n");
 		g_fprintf(fp,"snd_delay=%llu\n",(unsigned long long) global->Sound_delay);
-		g_fprintf(fp,"# Sound Format (PCM=1 (0x0001) MP2=80 (0x0050)\n");
-		g_fprintf(fp,"snd_format=%i\n",global->Sound_Format);
-		//g_fprintf(fp,"#Pan Step in degrees, Default=2\n");
-		//g_fprintf(fp,"Pan_Step=%i\n",global->PanStep);
-		//g_fprintf(fp,"#Tilt Step in degrees, Default=2\n");
-		//g_fprintf(fp,"Tilt_Step=%i\n",global->TiltStep);
-		g_fprintf(fp,"# video filters: 0 -none 1- flip 2- upturn 4- negate 8- mono (add the ones you want)\n");
+		g_fprintf(fp,"# Audio codec (PCM=0; MPG2=1; ... )\n");
+		g_fprintf(fp,"aud_codec=%i\n",global->AudCodec);
+		g_fprintf(fp,"# video filters: 0 -none 1- flip 2- upturn 4- negate 8- mono ...\n");
 		g_fprintf(fp,"frame_flags=%i\n",global->Frame_Flags);
 		g_fprintf(fp,"#on screen display flags (VU meter)\n");
 		g_fprintf(fp,"osd_flags=%i\n",global->osdFlags);
 		g_fprintf(fp,"# Auto Image naming (filename-n.ext)\n");
-		g_fprintf(fp,"image_inc=%d\n",global->image_inc);
+		g_fprintf(fp,"image_inc=%i\n",(global->image_inc > 0 ? 1: 0));
 		g_fprintf(fp,"# Image capture Full Path\n");
 		g_fprintf(fp,"image_path='%s/%s'\n",global->imgFPath[1],global->imgFPath[0]);
 		g_fprintf(fp,"# Video capture Full Path\n");
@@ -152,36 +141,36 @@ writeConf(struct GLOBAL *global, char *videodevice)
 		g_fprintf(fp, "vcodec_num_threads=%d\n",vcodec_defaults->num_threads);
 		g_fprintf(fp, "vcodec_flags=%d\n",vcodec_defaults->flags);
 		printf("write %s OK\n",global->confPath);
-		
+
 		//flush stream buffers to filesystem
 		fflush(fp);
-		
-		//close file after fsync (sync file data to disk) 
+
+		//close file after fsync (sync file data to disk)
 		if (fsync(fileno(fp)) || fclose(fp))
 		{
 			perror("error writing configuration data to file");
 			ret=-1;
-		} 
+		}
 		else
 		{
 			//rename from tmp to real name
 			g_rename (tmpfile, global->confPath);
 		}
-	} 
-	else 
+	}
+	else
 	{
 		g_printerr("Could not write file %s \n Please check file permissions\n",tmpfile);
 		ret=-2;
 	}
 	g_free(tmpfile);
-    
+
     //return to system locale
     setlocale(LC_NUMERIC, "");
-    
+
 	return ret;
 }
 
-/*----------------------- read conf (.guvcviewrc(-videoX)) file -----------------------*/
+/*----------------------- read conf (.config/guvcview/videoX) file -----------------------*/
 int
 readConf(struct GLOBAL *global)
 {
@@ -189,7 +178,7 @@ readConf(struct GLOBAL *global)
 	//int signal=1; /*1=>+ or -1=>-*/
 	GScanner  *scanner;
 	GTokenType ttype;
-	GScannerConfig config = 
+	GScannerConfig config =
 	{
 		" \t\r\n",                     /* characters to skip */
 		G_CSET_a_2_z "_" G_CSET_A_2_Z, /* identifier start */
@@ -218,9 +207,9 @@ readConf(struct GLOBAL *global)
 		FALSE,                         /* scope 0 fallback */
 		TRUE                           /* store int64 */
 	};
-    
+
 	int fd = g_open (global->confPath, O_RDONLY, 0);
-	
+
 	if (fd < 0 )
 	{
 		printf("Could not open %s for read,\n will try to create it\n",global->confPath);
@@ -237,18 +226,18 @@ readConf(struct GLOBAL *global)
 		int vc_gop_size=-1, vc_subq=-1, vc_framerefs=-1, vc_mb_decision=-1, vc_trellis=-1, vc_me_method=-1;
 		int vc_mpeg_quant=-1, vc_max_b_frames=-1, vc_num_threads=-1, vc_flags=-1, vc_monotonic_pts=-1;
 		float vc_qcompress=-1, vc_qblur=-1;
-		int VMAJOR =-1, VMINOR=-1, VMICRO=-1; 
-		
+		int VMAJOR =-1, VMINOR=-1, VMICRO=-1;
+
 		for (ttype = g_scanner_get_next_token (scanner);
 			ttype != G_TOKEN_EOF;
-			ttype = g_scanner_get_next_token (scanner)) 
+			ttype = g_scanner_get_next_token (scanner))
 		{
-			if (ttype == G_TOKEN_STRING) 
+			if (ttype == G_TOKEN_STRING)
 			{
 				//printf("reading %s...\n",scanner->value.v_string);
 				char *name = g_strdup (scanner->value.v_string);
 				ttype = g_scanner_get_next_token (scanner);
-				if (ttype != G_TOKEN_EQUAL_SIGN) 
+				if (ttype != G_TOKEN_EQUAL_SIGN)
 				{
 					g_scanner_unexp_token (scanner,
 						G_TOKEN_EQUAL_SIGN,
@@ -267,30 +256,25 @@ readConf(struct GLOBAL *global)
 						//signal = -1;
 						ttype = g_scanner_get_next_token (scanner);
 					}
-					
+
 					if (ttype == G_TOKEN_STRING)
 					{
-						//signal=1; /*reset signal*/
-						//if (g_strcmp0(name,"video_device")==0) 
-						//{
-						//	g_snprintf(global->videodevice,15,"%s",scanner->value.v_string);
-						//}
-						
+
 						if (g_strcmp0(name,"version")==0)
 						{
 							sscanf(scanner->value.v_string,"%i.%i.%i",
-								&(VMAJOR), 
+								&(VMAJOR),
 								&(VMINOR),
 								&(VMICRO));
 						}
-						else if (g_strcmp0(name,"resolution")==0) 
+						else if (g_strcmp0(name,"resolution")==0)
 						{
 							if(global->flg_res < 1) /*must check for defaults since ReadOpts runs before ReadConf*/
 								sscanf(scanner->value.v_string,"%ix%i",
-									&(global->width), 
+									&(global->width),
 									&(global->height));
 						}
-						else if (g_strcmp0(name,"windowsize")==0) 
+						else if (g_strcmp0(name,"windowsize")==0)
 						{
 							sscanf(scanner->value.v_string,"%ix%i",
 								&(global->winwidth), &(global->winheight));
@@ -317,7 +301,7 @@ readConf(struct GLOBAL *global)
 							{
 								global->imgFPath = splitPath(scanner->value.v_string,global->imgFPath);
 								/*get the file type*/
-		
+
 								global->imgFormat = check_image_type(global->imgFPath[0]);
 							}
 							else
@@ -325,29 +309,27 @@ readConf(struct GLOBAL *global)
 							    /* check if new file != old file */
 							    gchar * newPath = g_strjoin ("/", global->imgFPath[1], global->imgFPath[0], NULL);
 							    //printf("image path: %s\n old path: %s\n", newPath, scanner->value.v_string);
-							    if(g_strcmp0(scanner->value.v_string, newPath) !=0) 
+							    if(g_strcmp0(scanner->value.v_string, newPath) !=0)
 	                            {
 	                                /* reset counter */
 	                                //printf("reset counter from: %i\n", global->image_inc);
-	                                if(global->image_inc > 0) 
+	                                if(global->image_inc > 0)
 	                                {
 	                                    global->image_inc = 1;
-	                                    g_snprintf(global->imageinc_str,20,_("File num:%d"),global->image_inc);  
+	                                    //g_snprintf(global->imageinc_str,20,_("File num:%d"),global->image_inc);
 	                                }
 	                            }
 	                            g_free(newPath);
 							}
 						}
-						else if ((g_strcmp0(name,"video_path")==0) || (g_strcmp0(name,"avi_path")==0)) 
+						else if ((g_strcmp0(name,"video_path")==0) || (g_strcmp0(name,"avi_path")==0))
 						{
 							if(global->vidfile == NULL)
 							{
 								global->vidFPath=splitPath(scanner->value.v_string,global->vidFPath);
-								/*get the file type (0-avi 1-matroska)*/
-								global->VidFormat = check_video_type(global->vidFPath[0]);
 							}
 						}
-						else if (g_strcmp0(name,"profile_path")==0) 
+						else if (g_strcmp0(name,"profile_path")==0)
 						{
 							if(global->lprofile < 1)
 								global->profile_FPath=splitPath(scanner->value.v_string,
@@ -355,17 +337,13 @@ readConf(struct GLOBAL *global)
 						}
 						else
 						{
-							printf("unexpected string value (%s) for %s\n", 
+							printf("unexpected string value (%s) for %s\n",
 								scanner->value.v_string, name);
 						}
 					}
 					else if (ttype==G_TOKEN_INT)
 					{
-						if (g_strcmp0(name,"stack_size")==0)
-						{
-							global->stack_size = scanner->value.v_int;
-						}
-						else if (g_strcmp0(name,"vid_sleep")==0)
+						if (g_strcmp0(name,"vid_sleep")==0)
 						{
 							global->vid_sleep = scanner->value.v_int;
 						}
@@ -374,14 +352,6 @@ readConf(struct GLOBAL *global)
 							if(!(global->flg_cap_meth))
 								global->cap_meth = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vpane")==0) 
-						{
-							global->boxvsize = scanner->value.v_int;
-						}
-						else if (g_strcmp0(name,"spinbehave")==0) 
-						{
-							global->spinbehave = scanner->value.v_int;
-						}
 						else if (g_strcmp0(name,"default_action")==0)
 						{
 							global->default_action = scanner->value.v_int;
@@ -390,19 +360,19 @@ readConf(struct GLOBAL *global)
 						{
 							/*parse non-quoted fps values*/
 							int line = g_scanner_cur_line(scanner);
-							
+
 							global->fps_num = scanner->value.v_int;
 							ttype = g_scanner_peek_next_token (scanner);
 							if(ttype=='/')
 							{
 								/*get '/'*/
-								ttype = g_scanner_get_next_token (scanner);
+								g_scanner_get_next_token (scanner);
 								ttype = g_scanner_peek_next_token (scanner);
 								if(ttype==G_TOKEN_INT)
 								{
 									ttype = g_scanner_get_next_token (scanner);
 									global->fps = scanner->value.v_int;
-								} 
+								}
 								else if (scanner->next_line>line)
 								{
 									/*start new loop*/
@@ -421,184 +391,167 @@ readConf(struct GLOBAL *global)
 								}
 							}
 						}
-						else if (strcmp(name,"fps_display")==0) 
+						else if (strcmp(name,"fps_display")==0)
 						{
 							if(global->flg_FpsCount < 1)
 								global->FpsCount = (short) scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"auto_focus")==0) 
+						else if (g_strcmp0(name,"auto_focus")==0)
 						{
 							global->autofocus = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"bpp")==0) 
+						else if (g_strcmp0(name,"bpp")==0)
 						{
 							global->bpp = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"hwaccel")==0) 
+						else if (g_strcmp0(name,"hwaccel")==0)
 						{
 							if(global->flg_hwaccel < 1)
 								global->hwaccel = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vid_codec")==0 || (g_strcmp0(name,"avi_format")==0)) 
+						else if (g_strcmp0(name,"vid_codec")==0 || (g_strcmp0(name,"avi_format")==0))
 						{
 							global->VidCodec = scanner->value.v_int;
+							//sync with global->VidCodec_ID when creating the menu entry for the codecs
 						}
-						else if (g_strcmp0(name,"avi_max_len")==0) 
+						else if (g_strcmp0(name,"vid_format")==0)
 						{
-							global->AVI_MAX_LEN = (ULONG) scanner->value.v_int;
-							global->AVI_MAX_LEN = AVI_set_MAX_LEN (global->AVI_MAX_LEN);
+							global->VidFormat = scanner->value.v_int;
 						}
-						else if ((g_strcmp0(name,"vid_inc")==0) || (g_strcmp0(name,"avi_inc")==0)) 
+						else if ((g_strcmp0(name,"vid_inc")==0) || (g_strcmp0(name,"avi_inc")==0))
 						{
 							global->vid_inc = (DWORD) scanner->value.v_int;
-							g_snprintf(global->vidinc_str,20,_("File num:%d"),global->vid_inc);
 						}
-						else if (g_strcmp0(name,"sound")==0) 
+						else if (g_strcmp0(name,"sound")==0)
 						{
 							global->Sound_enable = (short) scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"snd_api")==0) 
+						else if (g_strcmp0(name,"snd_api")==0)
 						{
 							global->Sound_API = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"snd_device")==0) 
+						else if (g_strcmp0(name,"snd_device")==0)
 						{
 							global->Sound_UseDev = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"snd_samprate")==0) 
+						else if (g_strcmp0(name,"snd_samprate")==0)
 						{
 							global->Sound_SampRateInd = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"snd_numchan")==0) 
+						else if (g_strcmp0(name,"snd_numchan")==0)
 						{
 							global->Sound_NumChanInd = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"snd_delay")==0) 
+						else if (g_strcmp0(name,"snd_delay")==0)
 						{
 							global->Sound_delay = scanner->value.v_int64;
 						}
-						else if (g_strcmp0(name,"snd_format")==0) 
+						else if (g_strcmp0(name,"aud_codec")==0)
 						{
-							global->Sound_Format = scanner->value.v_int;
+							global->AudCodec = scanner->value.v_int;
+							global->Sound_Format = get_aud4cc(global->AudCodec);
 						}
-						// else if (g_strcmp0(name,"snd_bitrate")==0) 
-						// {
-							// global->Sound_bitRate = scanner->value.v_int;
-						// }
-						//else if (g_strcmp0(name,"Pan_Step")==0)
-						//{
-						//	global->PanStep = signal * scanner->value.v_int;
-						//	signal = 1; /*reset signal*/
-						//}
-						//else if (g_strcmp0(name,"Tilt_Step")==0)
-						//{
-						//	global->TiltStep = signal * scanner->value.v_int;
-						//	signal = 1; /*reset signal*/
-						//}
-						else if (g_strcmp0(name,"frame_flags")==0) 
+						else if (g_strcmp0(name,"frame_flags")==0)
 						{
 							global->Frame_Flags = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"osd_flags")==0) 
+						else if (g_strcmp0(name,"osd_flags")==0)
 						{
 							global->osdFlags = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"image_inc")==0) 
+						else if (g_strcmp0(name,"image_inc")==0)
 						{
 							global->image_inc = (DWORD) scanner->value.v_int;
-							//if((global->image_timer > 0) && (global->image_inc <= 0))
-							//    global->image_inc = 1;
-							g_snprintf(global->imageinc_str,20,_("File num:%d"),global->image_inc);
 						}
-						else if (g_strcmp0(name,"acodec_bit_rate")==0) 
+						else if (g_strcmp0(name,"acodec_bit_rate")==0)
 						{
 						    ac_bit_rate = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_bit_rate")==0) 
+						else if (g_strcmp0(name,"vcodec_bit_rate")==0)
 						{
 						    vc_bit_rate = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_fps")==0) 
+						else if (g_strcmp0(name,"vcodec_fps")==0)
 						{
 						    vc_fps = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_monotonic_pts")==0) 
+						else if (g_strcmp0(name,"vcodec_monotonic_pts")==0)
 						{
 						    vc_monotonic_pts = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_qmax")==0) 
+						else if (g_strcmp0(name,"vcodec_qmax")==0)
 						{
 						    vc_qmax = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_qmin")==0) 
+						else if (g_strcmp0(name,"vcodec_qmin")==0)
 						{
 						    vc_qmin = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_max_qdiff")==0) 
+						else if (g_strcmp0(name,"vcodec_max_qdiff")==0)
 						{
 						    vc_max_qdiff = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_dia")==0) 
+						else if (g_strcmp0(name,"vcodec_dia")==0)
 						{
 						    vc_dia = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_pre_dia")==0) 
+						else if (g_strcmp0(name,"vcodec_pre_dia")==0)
 						{
 						    vc_pre_dia = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_pre_me")==0) 
+						else if (g_strcmp0(name,"vcodec_pre_me")==0)
 						{
 						    vc_pre_me= scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_me_pre_cmp")==0) 
+						else if (g_strcmp0(name,"vcodec_me_pre_cmp")==0)
 						{
 						    vc_me_pre_cmp = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_me_cmp")==0) 
+						else if (g_strcmp0(name,"vcodec_me_cmp")==0)
 						{
 						    vc_me_cmp = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_me_sub_cmp")==0) 
+						else if (g_strcmp0(name,"vcodec_me_sub_cmp")==0)
 						{
 						    vc_me_sub_cmp = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_last_pred")==0) 
+						else if (g_strcmp0(name,"vcodec_last_pred")==0)
 						{
 						    vc_last_pred = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_gop_size")==0) 
+						else if (g_strcmp0(name,"vcodec_gop_size")==0)
 						{
 						    vc_gop_size = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_subq")==0) 
+						else if (g_strcmp0(name,"vcodec_subq")==0)
 						{
 						    vc_subq = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_framerefs")==0) 
+						else if (g_strcmp0(name,"vcodec_framerefs")==0)
 						{
 						    vc_framerefs = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_mb_decision")==0) 
+						else if (g_strcmp0(name,"vcodec_mb_decision")==0)
 						{
 						    vc_mb_decision = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_trellis")==0) 
+						else if (g_strcmp0(name,"vcodec_trellis")==0)
 						{
 						    vc_trellis = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_me_method")==0) 
+						else if (g_strcmp0(name,"vcodec_me_method")==0)
 						{
 						    vc_me_method = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_mpeg_quant")==0) 
+						else if (g_strcmp0(name,"vcodec_mpeg_quant")==0)
 						{
 						    vc_mpeg_quant = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_max_b_frames")==0) 
+						else if (g_strcmp0(name,"vcodec_max_b_frames")==0)
 						{
 						    vc_max_b_frames = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vcodec_flags")==0) 
+						else if (g_strcmp0(name,"vcodec_flags")==0)
 						{
 						    vc_flags = scanner->value.v_int;
 						}
@@ -608,18 +561,18 @@ readConf(struct GLOBAL *global)
 						}
 						else
 						{
-							printf("unexpected integer value (%lu) for %s\n", 
+							printf("unexpected integer value (%lu) for %s\n",
 								scanner->value.v_int, name);
 							printf("Strings must be quoted\n");
 						}
 					}
 					else if (ttype==G_TOKEN_FLOAT)
 					{
-					    if (g_strcmp0(name,"vcodec_qcompress")==0) 
+					    if (g_strcmp0(name,"vcodec_qcompress")==0)
 						{
 						    vc_qcompress = scanner->value.v_float;
 						}
-						else if (g_strcmp0(name,"vcodec_qblur")==0) 
+						else if (g_strcmp0(name,"vcodec_qblur")==0)
 						{
 						    vc_qblur = scanner->value.v_float;
 						}
@@ -641,7 +594,7 @@ readConf(struct GLOBAL *global)
 							FALSE);
 						int line = g_scanner_cur_line (scanner);
 						int stp=0;
-					
+
 						do
 						{
 							ttype = g_scanner_peek_next_token (scanner);
@@ -662,14 +615,14 @@ readConf(struct GLOBAL *global)
 				g_free(name);
 			}
 		}
-		
+
 		g_scanner_destroy (scanner);
 		close (fd);
-		
+
 		//get pointers to codec properties
         vcodecs_data *vcodec_defaults = get_codec_defaults(global->VidCodec);
         acodecs_data *acodec_defaults = get_aud_codec_defaults(get_ind_by4cc(global->Sound_Format));
-        
+
         if (ac_bit_rate >= 0) acodec_defaults->bit_rate = ac_bit_rate;
         if (vc_bit_rate >= 0) vcodec_defaults->bit_rate = vc_bit_rate;
         if (vc_fps >= 0) vcodec_defaults->fps = vc_fps;
@@ -697,38 +650,61 @@ readConf(struct GLOBAL *global)
         if (vc_flags >=0) vcodec_defaults->flags = vc_flags;
         if (vc_qcompress >= 0) vcodec_defaults->qcompress = vc_qcompress;
 		if (vc_qblur >=0) vcodec_defaults->qblur = vc_qblur;
-        
-		if (global->debug) 
+
+        if(global->vid_inc>0)
+		{
+			uint64_t suffix = get_file_suffix(global->vidFPath[1], global->vidFPath[0]);
+			fprintf(stderr, "Video file suffix detected: %" PRIu64 "\n", suffix);
+			if(suffix >= G_MAXUINT64)
+			{
+				global->vidFPath[0] = add_file_suffix(global->vidFPath[0], suffix);
+				suffix = 0;
+			}
+			if(suffix > 0)
+				global->vid_inc = suffix + 1;
+		}
+
+		if(global->image_inc>0)
+		{
+			uint64_t suffix = get_file_suffix(global->imgFPath[1], global->imgFPath[0]);
+			fprintf(stderr, "Image file suffix detected: %" PRIu64 "\n", suffix);
+			if(suffix >= G_MAXUINT64)
+			{
+				global->imgFPath[0] = add_file_suffix(global->imgFPath[0], suffix);
+				suffix = 0;
+			}
+			if(suffix > 0)
+				global->image_inc = suffix + 1;
+		}
+
+		if (global->debug)
 		{
 			g_print("video_device: %s\n",global->videodevice);
 			g_print("vid_sleep: %i\n",global->vid_sleep);
 			g_print("cap_meth: %i\n",global->cap_meth);
 			g_print("resolution: %i x %i\n",global->width,global->height);
 			g_print("windowsize: %i x %i\n",global->winwidth,global->winheight);
-			g_print("vert pane: %i\n",global->boxvsize);
-			g_print("spin behavior: %i\n",global->spinbehave);
 			g_print("default action: %i\n",global->default_action);
 			g_print("mode: %s\n",global->mode);
 			g_print("fps: %i/%i\n",global->fps_num,global->fps);
 			g_print("Display Fps: %i\n",global->FpsCount);
 			g_print("bpp: %i\n",global->bpp);
 			g_print("hwaccel: %i\n",global->hwaccel);
-			g_print("avi_format: %i\n",global->VidCodec);
+			g_print("vid_codec: %i\n",global->VidCodec);
 			g_print("sound: %i\n",global->Sound_enable);
 			g_print("sound Device: %i\n",global->Sound_UseDev);
 			g_print("sound samp rate: %i\n",global->Sound_SampRateInd);
 			g_print("sound Channels: %i\n",global->Sound_NumChanInd);
 			g_print("Sound delay: %llu nanosec\n",(unsigned long long) global->Sound_delay);
 			g_print("Sound Format: %i \n",global->Sound_Format);
-			//g_print("Sound bit Rate: %i Kbps\n",global->Sound_bitRate);
 			g_print("Pan Step: %i degrees\n",global->PanStep);
 			g_print("Tilt Step: %i degrees\n",global->TiltStep);
 			g_print("Video Filter Flags: %i\n",global->Frame_Flags);
-			g_print("image inc: %d\n",global->image_inc);
+			g_print("image inc: %" PRIu64 "\n",global->image_inc);
 			g_print("profile(default):%s/%s\n",global->profile_FPath[1],global->profile_FPath[0]);
 		}
 	}
-	
+
 	return (ret);
 }
 
@@ -754,7 +730,7 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 	int hwaccel=-1;
 	int FpsCount=-1;
 	int cap_meth=-1;
-	
+
 	GOptionEntry entries[] =
 	{
 		{ "help-all", 'h', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &help_all, "Display all help options", NULL},
@@ -774,12 +750,13 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 		{ "image", 'i', 0, G_OPTION_ARG_STRING, &image, N_("Image File name"), "FILENAME"},
 		{ "cap_time", 'c', 0, G_OPTION_ARG_INT, &global->image_timer, N_("Image capture interval in seconds"), "TIME"},
 		{ "npics", 'm', 0, G_OPTION_ARG_INT, &global->image_npics, N_("Number of Pictures to capture"), "NUMPIC"},
-		{ "video", 'n', 0, G_OPTION_ARG_STRING, &video, N_("Video File name (capture from start)"), "FILENAME"},
-		{ "vid_time", 't', 0, G_OPTION_ARG_INT, &global->Capture_time,N_("Video capture time (in seconds)"), "TIME"},
+		{ "video", 'n', 0, G_OPTION_ARG_STRING, &video, N_("Video File name"), "FILENAME"},
+		{ "vid_time", 't', 0, G_OPTION_ARG_INT, &global->Capture_time,N_("Video capture time (seconds) - capture from start"), "TIME"},
 		{ "exit_on_close", 0, 0, G_OPTION_ARG_NONE, &global->exit_on_close, N_("Exits guvcview after closing video"), NULL},
 		{ "skip", 'j', 0, G_OPTION_ARG_INT, &global->skip_n, N_("Number of initial frames to skip"), "N_FRAMES"},
 		{ "show_fps", 'p', 0, G_OPTION_ARG_INT, &FpsCount, N_("Show FPS value (enable(1) | disable (0))"), "[1 | 0]"},
 		{ "profile", 'l', 0, G_OPTION_ARG_STRING, &profile, N_("Load Profile at start"), "FILENAME"},
+		{ "lctl_method", 'k', 0, G_OPTION_ARG_INT, &global->lctl_method, N_("List controls method (0:loop, 1:next_ctrl flag [def])"), "[0 |1]"},
 		{ NULL }
 	};
 
@@ -787,7 +764,7 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 	GOptionContext *context;
 	context = g_option_context_new (N_("- local options"));
 	g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
-	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	g_option_context_add_group (context, gtk_get_option_group (FALSE));
 	g_set_prgname (PACKAGE);
 	help_str = g_option_context_get_help (context, TRUE, NULL);
 	help_gtk_str = g_option_context_get_help (context, FALSE, gtk_get_option_group (TRUE));
@@ -807,7 +784,7 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 		g_option_context_free (context);
 		exit (1);
 	}
-	
+
 	if(vers)
 	{
 		//print version and exit
@@ -853,7 +830,7 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 		g_option_context_free (context);
 		exit(0);
 	}
-	
+
 	/*regular options*/
 	if(device)
 	{
@@ -875,7 +852,7 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 				g_free(dirname);
 				dirname=g_strdup("/dev");
 			}
-		
+
 			global->videodevice = g_strjoin("/",
 				dirname,
 				basename,
@@ -886,9 +863,10 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 				{
 					g_free(global->confPath);
 					global->confPath=NULL;
-					global->confPath = g_strjoin("", 
-						g_get_home_dir(), 
-						"/.guvcviewrc-",
+					global->confPath = g_strjoin("/",
+						g_get_home_dir(),
+						".config",
+						"guvcview",
 						basename,
 						NULL);
 				}
@@ -913,26 +891,26 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 			g_snprintf(global->mode,5,"ba81");
 		else
 			g_snprintf(global->mode,5,"%s ",format);
-			
+
 	    printf("requested format \"%s\" from command line\n", global->mode);
-		
+
 		global->flg_mode = TRUE;
 	}
 	if(size)
 	{
 		global->width = (int) g_ascii_strtoull(size, &separateur, 10);
-		if (*separateur != 'x') 
+		if (*separateur != 'x')
 		{
 			g_printerr("Error in size usage: -s[--size] WIDTHxHEIGHT \n");
-		} 
-		else 
+		}
+		else
 		{
 			++separateur;
 			global->height = (int) g_ascii_strtoull(separateur, &separateur, 10);
 			if (*separateur != 0)
 				g_printerr("hmm.. don't like that!! trying this height \n");
 		}
-		
+
 		global->flg_res = 1;
 	}
 	if(image)
@@ -941,6 +919,19 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 		/*get the file type*/
 		global->imgFormat = check_image_type(global->imgFPath[0]);
 		global->flg_imgFPath = TRUE;
+
+		if(global->image_inc>0)
+		{
+			uint64_t suffix = get_file_suffix(global->imgFPath[1], global->imgFPath[0]);
+			fprintf(stderr, "Image file suffix detected: %" PRIu64 "\n", suffix);
+			if(suffix >= G_MAXUINT64)
+			{
+				global->imgFPath[0] = add_file_suffix(global->imgFPath[0], suffix);
+				suffix = 0;
+			}
+			if(suffix > 0)
+				global->image_inc = suffix + 1;
+		}
 	}
 	if(global->image_timer > 0 )
 	{
@@ -948,9 +939,23 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 	}
 	if(video)
 	{
-		global->vidfile = g_strdup(video);
-		global->vidFPath=splitPath(global->vidfile,global->vidFPath);
-		g_print("capturing video: %s , from start\n",global->vidfile);
+		global->vidFPath=splitPath(video, global->vidFPath);
+		if(global->vid_inc>0)
+		{
+			uint64_t suffix = get_file_suffix(global->vidFPath[1], global->vidFPath[0]);
+			fprintf(stderr, "Video file suffix detected: %" PRIu64 "\n", suffix);
+			if(suffix >= G_MAXUINT64)
+			{
+				global->vidFPath[0] = add_file_suffix(global->vidFPath[0], suffix);
+				suffix = 0;
+			}
+			if(suffix > 0)
+				global->vid_inc = suffix + 1;
+		}
+
+		global->vidfile = joinPath(global->vidfile, global->vidFPath);
+
+		g_print("set video file: %s \n",global->vidfile);
 		/*get the file type*/
 		global->VidFormat = check_video_type(global->vidFPath[0]);
 	}
@@ -974,7 +979,7 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 		global->flg_cap_meth = TRUE;
 		global->cap_meth = cap_meth;
 	}
-	
+
 	//g_print("option capture meth is %i\n", global->cap_meth);
 	g_free(help_str);
 	g_free(help_gtk_str);
