@@ -73,11 +73,11 @@ struct paRecordData
 {
 	int api; //0-Portaudio 1-pulse audio
 	int input_type; // audio SAMPLE type
-	
+
 	unsigned long framesPerBuffer;   //frames per buffer passed in audio callback
 	char device_name[512];           //device name - for pulse
 	int device_id;                   //device id - for portaudio
-	
+
 	int w_ind;                       // producer index
 	int r_ind;                       // consumer index
 	int bw_ind;                      // audio_buffer in_use index
@@ -92,6 +92,7 @@ struct paRecordData
 	int aud_numBytes;                // bytes copied to out buffer*/
 	int aud_numSamples;              // samples copied to out buffer*/
 	int64_t snd_begintime;           // audio recording start time*/
+	//int64_t api_ts_ref;
 	int capVid;                      // video capture flag
 	SAMPLE *recordedSamples;         // callback buffer
 	int sampleIndex;
@@ -99,21 +100,23 @@ struct paRecordData
 	int audio_buff_flag[AUDBUFF_NUM];// process_buffer flags
 	int64_t a_ts;                    // audio frame time stamp
 	int64_t ts_ref;                  // timestamp video reference
+	int64_t a_last_ts;               // last audio frame timestamp
 	int64_t ts_drift;                // time drift between audio device clock and system clock
 	gint16 *pcm_sndBuff;             // buffer for pcm coding with int16
+	float  *float_sndBuff;           // buffer for pcm coding with float
 	BYTE *mp2Buff;                   // mp2 encode buffer
 	int mp2BuffSize;                 // mp2 buffer size
 	int snd_Flags;                   // effects flag
 	int skip_n;                      // video frames to skip
 	UINT64 delay;                    // in nanosec - h264 has a two frame delay that must be compensated
-	
+
 	int outbuf_size;	             //size of output buffer
 	struct lavcAData* lavc_data;     //libavcodec data
 	__MUTEX_TYPE mutex;
-	
+
 	//PORTAUDIO SUPPORT
 	void *stream;
-	
+
 	//PULSE SUPPORT
 #ifdef PULSEAUDIO
 	 __THREAD_TYPE pulse_read_th;
@@ -122,11 +125,11 @@ struct paRecordData
 #endif
 };
 
-int 
-record_sound ( const void *inputBuffer, unsigned long numSamples, void *userData );
+int
+record_sound ( const void *inputBuffer, unsigned long numSamples, int64_t timestamp, void *userData );
 
-int 
-fill_audio_buffer(struct paRecordData *pdata, UINT64 ts);
+void
+record_silence ( unsigned long numSamples, void *userData );
 
 void
 set_sound (struct GLOBAL *global, struct paRecordData* data);
@@ -137,8 +140,8 @@ init_sound(struct paRecordData* data);
 int
 close_sound (struct paRecordData *data);
 
-void 
-Float2Int16 (struct paRecordData* data);
+void
+SampleConverter (struct paRecordData* data);
 
 #endif
 
