@@ -1,5 +1,5 @@
 /*******************************************************************************#
-#           guvcview              http://guvcview.berlios.de                    #
+#           guvcview              http://guvcview.sourceforge.net               #
 #                                                                               #
 #           Paulo Assis <pj.assis@gmail.com>                                    #
 #                                                                               #
@@ -24,10 +24,16 @@
 
 #include <linux/videodev2.h>
 #include <libudev.h>
+#include <pthread.h>
 #include "globals.h"
 #include "v4l2_devices.h"
 #include "v4l2_formats.h"
 #include "v4l2_controls.h"
+#include "lavc_common.h"
+
+//control list method
+#define LIST_CTL_METHOD_LOOP 0
+#define LIST_CTL_METHOD_NEXT_FLAG 1
 
 #define NB_BUFFER 4
 
@@ -55,10 +61,10 @@
 
 #define HEADERFRAME1 0xaf
 
-//set ioctl retries to 4 - linux uvc as increased timeout from 1000 to 3000 ms  
+//set ioctl retries to 4 - linux uvc as increased timeout from 1000 to 3000 ms
 #define IOCTL_RETRY 4
 
-enum  v4l2_uvc_exposure_auto_type 
+enum  v4l2_uvc_exposure_auto_type
 {
 	V4L2_UVC_EXPOSURE_MANUAL = 1,
 	V4L2_UVC_EXPOSURE_AUTO = 2,
@@ -70,13 +76,50 @@ static const int exp_vals[]=
 {
 	V4L2_UVC_EXPOSURE_MANUAL,
 	V4L2_UVC_EXPOSURE_AUTO,
-	V4L2_UVC_EXPOSURE_SHUTTER_PRIORITY, 
+	V4L2_UVC_EXPOSURE_SHUTTER_PRIORITY,
 	V4L2_UVC_EXPOSURE_APERTURE_PRIORITY
 };
 
-struct vdIn 
+/* h264 probe commit struct */
+typedef struct _uvcx_video_config_probe_commit_t
 {
-	GMutex *mutex;                      // VdIn struct mutex
+	DWORD	dwFrameInterval;
+	DWORD	dwBitRate;
+	WORD	bmHints;
+	WORD	wConfigurationIndex;
+	WORD	wWidth;
+	WORD	wHeight;
+	WORD	wSliceUnits;
+	WORD	wSliceMode;
+	WORD	wProfile;
+	WORD	wIFramePeriod;
+	WORD	wEstimatedVideoDelay;
+	WORD	wEstimatedMaxConfigDelay;
+	BYTE	bUsageType;
+	BYTE	bRateControlMode;
+	BYTE	bTemporalScaleMode;
+	BYTE	bSpatialScaleMode;
+	BYTE	bSNRScaleMode;
+	BYTE	bStreamMuxOption;
+	BYTE	bStreamFormat;
+	BYTE	bEntropyCABAC;
+	BYTE	bTimestamp;
+	BYTE	bNumOfReorderFrames;
+	BYTE	bPreviewFlipped;
+	BYTE	bView;
+	BYTE	bReserved1;
+	BYTE	bReserved2;
+	BYTE	bStreamID;
+	BYTE	bSpatialLayerRatio;
+	WORD	wLeakyBucketSize;
+} __attribute__((__packed__)) uvcx_video_config_probe_commit_t;
+
+struct vdIn
+{
+	__MUTEX_TYPE mutex;
+
+	uint64_t frame_index;				// captured frame index
+
     struct udev *udev;                  // pointer to a udev struct (lib udev)
     struct udev_monitor *udev_mon;      // udev monitor
     int udev_fd;                        // udev monitor file descriptor
@@ -90,30 +133,42 @@ struct vdIn
 	//struct v4l2_timecode timecode;      // v4l2 timecode struct
 	struct v4l2_streamparm streamparm;  // v4l2 stream parameters struct
 	struct v4l2_jpegcompression jpgcomp;// v4l2 jpeg compression settings
+	uvcx_video_config_probe_commit_t h264_config_probe_req; //probe commit struct for h264 streams
 	void *mem[NB_BUFFER];               // memory buffers for mmap driver frames
 	UINT32 buff_length[NB_BUFFER];      // memory buffers length as set by VIDIOC_QUERYBUF
 	UINT32 buff_offset[NB_BUFFER];      // memory buffers offset as set by VIDIOC_QUERYBUF
-	unsigned char *tmpbuffer;           // temp buffer for decoding compressed data
-	unsigned char *framebuffer;         // frame buffer (YUYV), for rendering in SDL overlay
+	uint8_t *tmpbuffer;                 // temp buffer for decoding compressed data
+	uint8_t *framebuffer;               // frame buffer (YUYV), for rendering in SDL overlay
+	gboolean isKeyframe;                // current buffer contains a keyframe (h264 IDR)
 	int isstreaming;                    // video stream flag (1- ON  0- OFF)
 	int isbayer;                        // raw bayer flag
 	int pix_order;                      // raw bayer pixel order (rg/gb, bg/gr, ...)
 	int setFPS;                         // set FPS flag (0-do nothing, 1-change fps value, 2-query and queue buffer)
 	int setJPEGCOMP;                    // set jpeg compression flag (0-do nothing, 1-change compression value, 2-query and queue buffer)
+	int setH264ConfigProbe;             // set H264 stream Config Probe Commit
 	int grabmethod;                     // only mmap available UVC doesn't support read
-	UINT64 timestamp;                   //video frame time stamp
+	UINT64 timestamp;                   // video frame time stamp
 	char *VidFName;                     // Video File name (with full path)
 	int capImage;                       // Image capture flag (raised for capturing a frame)
 	char *ImageFName;                   // Image File name (with full path)
 	int cap_raw;                        // raw frame capture flag
-	int available_exp[4];               //backward compatible (old v4l2 exposure menu interface)
-	int PanTilt;                        //1-if PanTilt Camera 0-otherwise
+	int available_exp[4];               // backward compatible (old v4l2 exposure menu interface)
+	int PanTilt;                        // 1-if PanTilt Camera 0-otherwise
+	int uvc_h264_unit;     				// uvc h264 unit id, if <= 0 then uvc h264 is not supported
 	gboolean signalquit;                // video loop exit flag
 	gboolean capVid;                    // Video capture flag (raised while capturing)
 	gboolean VidCapStop;                // Video capture stop flag (raised when video capture has stopped)
 	gboolean IOfinished;                // Signals that IO thread has finished
 	LFormats *listFormats;              // structure with frame formats list
 	LDevices *listDevices;              // structure with devices list
+	struct h264_decoder_context* h264_ctx; //h264 decoder context
+	uint8_t *h264_last_IDR;             // last IDR frame retrieved from uvc h264 stream
+	int h264_last_IDR_size;             // last IDR frame size
+	uint8_t *h264_frame;                // current uvc h264 frame retrieved from video stream
+	uint8_t *h264_SPS;                  // h264 SPS info
+	uint16_t h264_SPS_size;             // SPS size
+	uint8_t *h264_PPS;                  // h264 PPS info
+	uint16_t h264_PPS_size;             // PPS size
 };
 
 /* ioctl with a number of retries in the case of I/O failure
@@ -137,12 +192,13 @@ int init_videoIn(struct vdIn *vd, struct GLOBAL *global);
 /* Grabs video frame and decodes it if necessary
  * args:
  * vd: pointer to a VdIn struct ( must be allready initiated)
- * format: pixel v4l2_format
- * width: frame width
- * height: frame height
+ * global: pointer to a GLOBAL struct ( must be allready initiated)
+ * format: pixel v4l2_format (don't use global format as it may change while capturing)
+ * width: frame width (don't use global width as it may change while capturing)
+ * height: frame height (don't use global height as it may change while capturing)
  * returns: error code ( 0 - VDIN_OK)
 */
-int uvcGrab(struct vdIn *vd, int format, int width, int height, int *fps, int *fps_num);
+int uvcGrab(struct vdIn *vd, struct GLOBAL *global, int format, int width, int height);
 
 /* cleans VdIn struct and allocations
  * args:
@@ -155,7 +211,7 @@ void close_v4l2(struct vdIn *vd, gboolean control_only);
 /* restarts v4l2 device with new format
  * args:
  * vd: pointer to vdIn struct
- * global: poiter to GLOBAL struct 
+ * global: poiter to GLOBAL struct
  *
  * returns: error code ( 0 - VDIN_OK)
 */
