@@ -5,7 +5,7 @@
 #           Nobuhiro Iwamatsu <iwamatsu@nigauri.org>                            #
 #                             Add UYVY color support(Macbook iSight)            #
 #           Flemming Frandsen <dren.dk@gmail.com>                               #
-#                             Add VU meter OSD                                  #  
+#                             Add VU meter OSD                                  #
 #                                                                               #
 # This program is free software; you can redistribute it and/or modify          #
 # it under the terms of the GNU General Public License as published by          #
@@ -52,51 +52,51 @@
 
 static Uint32 SDL_VIDEO_Flags =
         SDL_ANYFORMAT | SDL_RESIZABLE;
-        
+
 static const SDL_VideoInfo *info;
 
 static SDL_Overlay * video_init(void *data, SDL_Surface **pscreen)
 {
     struct ALL_DATA *all_data = (struct ALL_DATA *) data;
     struct GLOBAL *global = all_data->global;
-    
+
     int width = global->width;
     int height = global->height;
-    
+
     if (*pscreen == NULL) //init SDL
     {
         char driver[128];
         /*----------------------------- Test SDL capabilities ---------------------*/
-        if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) < 0) 
+        if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER) < 0)
         {
             g_printerr("Couldn't initialize SDL: %s\n", SDL_GetError());
             exit(1);
         }
-    
+
         /* For this version, we will use hardware acceleration as default*/
-        if(global->hwaccel) 
+        if(global->hwaccel)
         {
             if ( ! getenv("SDL_VIDEO_YUV_HWACCEL") ) putenv("SDL_VIDEO_YUV_HWACCEL=1");
-            if ( ! getenv("SDL_VIDEO_YUV_DIRECT") ) putenv("SDL_VIDEO_YUV_DIRECT=1"); 
+            if ( ! getenv("SDL_VIDEO_YUV_DIRECT") ) putenv("SDL_VIDEO_YUV_DIRECT=1");
         }
-        else 
+        else
         {
             if ( ! getenv("SDL_VIDEO_YUV_HWACCEL") ) putenv("SDL_VIDEO_YUV_HWACCEL=0");
-            if ( ! getenv("SDL_VIDEO_YUV_DIRECT") ) putenv("SDL_VIDEO_YUV_DIRECT=0"); 
+            if ( ! getenv("SDL_VIDEO_YUV_DIRECT") ) putenv("SDL_VIDEO_YUV_DIRECT=0");
         }
-     
-        if (SDL_VideoDriverName(driver, sizeof(driver)) && global->debug) 
+
+        if (SDL_VideoDriverName(driver, sizeof(driver)) && global->debug)
         {
             g_print("Video driver: %s\n", driver);
         }
-    
+
         info = SDL_GetVideoInfo();
 
         if (info->wm_available && global->debug) g_print("A window manager is available\n");
 
-        if (info->hw_available) 
+        if (info->hw_available)
         {
-            if (global->debug) 
+            if (global->debug)
                 g_print("Hardware surfaces are available (%dK video memory)\n", info->video_mem);
 
             SDL_VIDEO_Flags |= SDL_HWSURFACE;
@@ -106,18 +106,18 @@ static SDL_Overlay * video_init(void *data, SDL_Surface **pscreen)
         {
             SDL_VIDEO_Flags |= SDL_SWSURFACE;
         }
-        
-        if (info->blit_hw) 
+
+        if (info->blit_hw)
         {
             if (global->debug) g_print("Copy blits between hardware surfaces are accelerated\n");
 
             SDL_VIDEO_Flags |= SDL_ASYNCBLIT;
         }
-        
+
         if(!global->desktop_w) global->desktop_w = info->current_w; //get desktop width
         if(!global->desktop_h) global->desktop_h = info->current_h; //get desktop height
-    
-        if (global->debug) 
+
+        if (global->debug)
         {
             if (info->blit_hw_CC) g_print ("Colorkey blits between hardware surfaces are accelerated\n");
             if (info->blit_hw_A) g_print("Alpha blits between hardware surfaces are accelerated\n");
@@ -127,13 +127,13 @@ static SDL_Overlay * video_init(void *data, SDL_Surface **pscreen)
             if (info->blit_fill) g_print("Color fills on hardware surfaces are accelerated\n");
         }
 
-        SDL_WM_SetCaption(global->WVcaption, NULL); 
+        SDL_WM_SetCaption(global->WVcaption, NULL);
 
         /* enable key repeat */
         SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
     }
     /*------------------------------ SDL init video ---------------------*/
-    if(global->debug)  
+    if(global->debug)
         g_print("(Desktop resolution = %ix%i)\n", global->desktop_w, global->desktop_h);
     g_print("Checking video mode %ix%i@32bpp : ", width, height);
     int bpp = SDL_VideoModeOK(
@@ -149,7 +149,7 @@ static SDL_Overlay * video_init(void *data, SDL_Surface **pscreen)
         if ((width > global->desktop_w) || (height > global->desktop_h))
         {
             width = global->desktop_w; /*use desktop video resolution*/
-            height = global->desktop_h; 
+            height = global->desktop_h;
         }
         else
         {
@@ -166,12 +166,12 @@ static SDL_Overlay * video_init(void *data, SDL_Surface **pscreen)
         global->bpp = bpp;
     }
 
-    *pscreen = SDL_SetVideoMode( 
+    *pscreen = SDL_SetVideoMode(
         width,
-        height, 
+        height,
         global->bpp,
         SDL_VIDEO_Flags);
-        
+
     if(*pscreen == NULL)
     {
         return (NULL);
@@ -185,42 +185,41 @@ static SDL_Overlay * video_init(void *data, SDL_Surface **pscreen)
     return (overlay);
 }
 
-/*-------------------------------- Main Video Loop ---------------------------*/ 
+/*-------------------------------- Main Video Loop ---------------------------*/
 /* run in a thread (SDL overlay)*/
 void *main_loop(void *data)
 {
     struct ALL_DATA *all_data = (struct ALL_DATA *) data;
-    
+
     struct VidState *s = all_data->s;
     struct paRecordData *pdata = all_data->pdata;
     struct GLOBAL *global = all_data->global;
     struct focusData *AFdata = all_data->AFdata;
     struct vdIn *videoIn = all_data->videoIn;
-    struct GWIDGET *gwidget = all_data->gwidget;
-    
+
     struct particle* particles = NULL; //for the particles video effect
-    
+
     SDL_Event event;
     /*the main SDL surface*/
     SDL_Surface *pscreen = NULL;
     SDL_Overlay *overlay = NULL;
     SDL_Rect drect;
-    
+
     int width = global->width;
     int height = global->height;
     int format = global->format;
-    
+
     SAMPLE vuPeak[2];  // The maximum vuLevel seen recently
     int vuPeakFreeze[2]; // The vuPeak values will be frozen for this many frames.
     vuPeak[0] = vuPeak[1] = 0;
     vuPeakFreeze[0] = vuPeakFreeze[1] = 0;
-    
+
     BYTE *p = NULL;
-    
+
     Control *focus_control = NULL;
     int last_focus = 0;
-    
-    if (global->AFcontrol) 
+
+    if (global->AFcontrol)
     {
         focus_control = get_ctrl_by_id(s->control_list, AFdata->id);
         get_ctrl(videoIn->fd, s->control_list, AFdata->id, all_data);
@@ -228,15 +227,15 @@ void *main_loop(void *data)
         /*make sure we wait for focus to settle on first check*/
         if (last_focus < 0) last_focus = AFdata->f_max;
     }
-    
+
     gboolean capVid = FALSE;
     gboolean signalquit = FALSE;
-    
+
     /*------------------------------ SDL init video ---------------------*/
     if(!global->no_display)
     {
         overlay = video_init(data, &(pscreen));
-        
+
         if(overlay == NULL)
         {
             g_print("FATAL: Couldn't create yuv overlay - please disable hardware accelaration\n");
@@ -245,45 +244,48 @@ void *main_loop(void *data)
         else
         {
             p = (unsigned char *) overlay->pixels[0];
-        
+
             drect.x = 0;
             drect.y = 0;
             drect.w = pscreen->w;
             drect.h = pscreen->h;
         }
     }
-    
-    while (!signalquit) 
+
+    while (!signalquit)
     {
         __LOCK_MUTEX(__VMUTEX);
             capVid = videoIn->capVid;
             signalquit = videoIn->signalquit;
         __UNLOCK_MUTEX(__VMUTEX);
-        
+
         /*-------------------------- Grab Frame ----------------------------------*/
-        if (uvcGrab(videoIn, format, width, height, &global->fps, &global->fps_num) < 0) 
+        if (uvcGrab(videoIn, global, format, width, height) < 0)
         {
             g_printerr("Error grabbing image \n");
             continue;
-        } 
-        else 
+        }
+        else
         {
             if(!videoIn->timestamp)
             {
                 global->skip_n++; //skip this frame
             }
-            /*reset video start time to first frame capture time */  
+
             if(capVid)
             {
                 if(global->framecount < 1)
                 {
-                    global->Vidstarttime = videoIn->timestamp;
-                    /*use current time for audio ts(0) reference (MONOTONIC)*/
-                    __LOCK_MUTEX(__AMUTEX);
-                        pdata->ts_ref = ns_time_monotonic();
-                    __UNLOCK_MUTEX(__AMUTEX);
-                    //printf("video ts ref: %llu audio ts_ ref: %llu\n",global->Vidstarttime, pdata->ts_ref);
-                    global->v_ts = 0;
+					/*reset video start time to first frame capture time */
+					global->Vidstarttime = videoIn->timestamp;
+					/** set current time for audio ts(0) reference (MONOTONIC)
+					 *  only used if we have no audio capture before video
+					 */
+					__LOCK_MUTEX(__AMUTEX);
+						pdata->ts_ref = ns_time_monotonic();
+					__UNLOCK_MUTEX(__AMUTEX);
+					//printf("video ts ref: %llu audio ts_ ref: %llu\n",global->Vidstarttime, pdata->ts_ref);
+					global->v_ts = 0;
                 }
                 else
                 {
@@ -292,60 +294,60 @@ void *main_loop(void *data)
                     global->Vidstoptime = videoIn->timestamp;
                 }
             }
-            
-            if (global->FpsCount && !global->no_display) 
+
+            if (global->FpsCount && !global->no_display)
             {/* sets fps count in window title bar */
                 global->frmCount++;
-                if (global->DispFps>0) 
+                if (global->DispFps>0)
                 { /*set every 2 sec*/
                     g_snprintf(global->WVcaption,24,"GUVCVideo - %3.2f fps",global->DispFps);
                     SDL_WM_SetCaption(global->WVcaption, NULL);
-                    
+
                     global->frmCount=0;/*resets*/
                     global->DispFps=0;
                 }
             }
-            
+
             /*---------------- autofocus control ------------------*/
-        
-            if (global->AFcontrol && (global->autofocus || AFdata->setFocus)) 
+
+            if (global->AFcontrol && (global->autofocus || AFdata->setFocus))
             { /*AFdata = NULL if no focus control*/
-                if (AFdata->focus < 0) 
+                if (AFdata->focus < 0)
                 {
                     /*starting autofocus*/
                     AFdata->focus = AFdata->left; /*start left*/
                     focus_control->value = AFdata->focus;
-                    if (set_ctrl (videoIn->fd, s->control_list, AFdata->id) != 0) 
+                    if (set_ctrl (videoIn->fd, s->control_list, AFdata->id) != 0)
                         g_printerr("ERROR: couldn't set focus to %d\n", AFdata->focus);
                     /*number of frames until focus is stable*/
                     /*1.4 ms focus time - every 1 step*/
                     AFdata->focus_wait = (int) abs(AFdata->focus-last_focus)*1.4/(1000/global->fps)+1;
                     last_focus = AFdata->focus;
-                } 
-                else 
+                }
+                else
                 {
-                    if (AFdata->focus_wait == 0) 
+                    if (AFdata->focus_wait == 0)
                     {
                         AFdata->sharpness=getSharpness (videoIn->framebuffer, width, height, 5);
-                        if (global->debug) 
+                        if (global->debug)
                             g_print("sharp=%d focus_sharp=%d foc=%d right=%d left=%d ind=%d flag=%d\n",
                                 AFdata->sharpness,AFdata->focus_sharpness,
-                                AFdata->focus, AFdata->right, AFdata->left, 
+                                AFdata->focus, AFdata->right, AFdata->left,
                                 AFdata->ind, AFdata->flag);
                         AFdata->focus=getFocusVal (AFdata);
-                        if ((AFdata->focus != last_focus)) 
+                        if ((AFdata->focus != last_focus))
                         {
                             focus_control->value = AFdata->focus;
-                            if (set_ctrl (videoIn->fd, s->control_list, AFdata->id) != 0) 
-                                g_printerr("ERROR: couldn't set focus to %d\n", 
+                            if (set_ctrl (videoIn->fd, s->control_list, AFdata->id) != 0)
+                                g_printerr("ERROR: couldn't set focus to %d\n",
                                     AFdata->focus);
                             /*number of frames until focus is stable*/
                             /*1.4 ms focus time - every 1 step*/
                             AFdata->focus_wait = (int) abs(AFdata->focus-last_focus)*1.4/(1000/global->fps)+1;
                         }
                         last_focus = AFdata->focus;
-                    } 
-                    else 
+                    }
+                    else
                     {
                         AFdata->focus_wait--;
                         if (global->debug) g_print("Wait Frame: %d\n",AFdata->focus_wait);
@@ -359,37 +361,37 @@ void *main_loop(void *data)
         {
             if((global->Frame_Flags & YUV_PARTICLES)==YUV_PARTICLES)
                 particles = particles_effect(videoIn->framebuffer, width, height, 20, 4, particles);
-            
-            if((global->Frame_Flags & YUV_MIRROR)==YUV_MIRROR) 
+
+            if((global->Frame_Flags & YUV_MIRROR)==YUV_MIRROR)
                 yuyv_mirror(videoIn->framebuffer, width, height);
-            
+
             if((global->Frame_Flags & YUV_UPTURN)==YUV_UPTURN)
                 yuyv_upturn(videoIn->framebuffer, width, height);
-                
+
             if((global->Frame_Flags & YUV_NEGATE)==YUV_NEGATE)
                 yuyv_negative (videoIn->framebuffer, width, height);
-                
-            if((global->Frame_Flags & YUV_MONOCR)==YUV_MONOCR) 
+
+            if((global->Frame_Flags & YUV_MONOCR)==YUV_MONOCR)
                 yuyv_monochrome (videoIn->framebuffer, width, height);
-           
+
             if((global->Frame_Flags & YUV_PIECES)==YUV_PIECES)
                 pieces (videoIn->framebuffer, width, height, 16 );
-            
+
         }
         __UNLOCK_MUTEX(__GMUTEX);
         /*-------------------------capture Image----------------------------------*/
         if (videoIn->capImage)
         {
-            /* 
+            /*
              * format and resolution can change(enabled) while capturing the frame
-             * but you would need to be speedy gonzalez to press two buttons 
-             * at almost the same time :D 
+             * but you would need to be speedy gonzalez to press two buttons
+             * at almost the same time :D
              */
             int ret = 0;
             if((ret=store_picture(all_data)) < 0)
                 g_printerr("saved image to:%s ...Failed \n",videoIn->ImageFName);
             else if (!ret && global->debug) g_print("saved image to:%s ...OK \n",videoIn->ImageFName);
-            
+
             videoIn->capImage=FALSE;
         }
         /*---------------------------capture Video---------------------------------*/
@@ -399,12 +401,12 @@ void *main_loop(void *data)
                 if(videoIn->VidCapStop) videoIn->VidCapStop = FALSE;
             __UNLOCK_MUTEX(__VMUTEX);
             int res=0;
-            
+
 			/* format and resolution don't change(disabled) while capturing video
 			 * store_video_frame may sleep if needed to avoid buffer overrun
 			 */
             if((res=store_video_frame(all_data))<0) g_printerr("WARNING: droped frame (%i)\n",res);
-            
+
         } /*video and audio capture have stopped */
         else
         {
@@ -423,7 +425,7 @@ void *main_loop(void *data)
         __LOCK_MUTEX( __AMUTEX );
             if (global->Sound_enable && capVid) pdata->skip_n = global->skip_n;
         __UNLOCK_MUTEX( __AMUTEX );
-        
+
         /*------------------------- Display Frame --------------------------------*/
         if(!global->no_display)
         {
@@ -435,15 +437,15 @@ void *main_loop(void *data)
             memcpy(p, videoIn->framebuffer, width * height * 2);
             SDL_UnlockYUVOverlay(overlay);
             SDL_DisplayYUVOverlay(overlay, &drect);
-        
+
             /*------------------------- Read Key events ------------------------------*/
             /* Poll for events */
             while( SDL_PollEvent(&event) )
             {
                 //printf("event type:%i  event key:%i\n", event.type, event.key.keysym.scancode);
-                if(event.type==SDL_KEYDOWN) 
+                if(event.type==SDL_KEYDOWN)
                 {
-                    if (videoIn->PanTilt) 
+                    if (videoIn->PanTilt)
                     {
                         switch( event.key.keysym.sym )
                         {
@@ -453,17 +455,17 @@ void *main_loop(void *data)
                                 /*Tilt Down*/
                                 uvcPanTilt (videoIn->fd, s->control_list, 0, 1);
                                 break;
-                                
+
                             case SDLK_UP:
                                 /*Tilt UP*/
                                 uvcPanTilt (videoIn->fd, s->control_list, 0, -1);
                                 break;
-                                
+
                             case SDLK_LEFT:
                                 /*Pan Left*/
                                 uvcPanTilt (videoIn->fd, s->control_list, 1, 1);
                                 break;
-                                
+
                             case SDLK_RIGHT:
                                 /*Pan Right*/
                                 uvcPanTilt (videoIn->fd, s->control_list, 1, -1);
@@ -475,16 +477,12 @@ void *main_loop(void *data)
                     switch( event.key.keysym.scancode )
                     {
                         case 220: /*webcam button*/
-                            gdk_threads_enter();
-                           	if (all_data->global->default_action == 0) 
-                           	{
-                           		gtk_button_clicked (GTK_BUTTON(gwidget->CapImageButt));
-                           	} 
-							else 
-							{
-                            	gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON(gwidget->CapVidButt));
-                            }
-                            gdk_threads_leave();
+                            //gdk_threads_enter();
+                           	if (all_data->global->default_action == 0)
+                           		g_main_context_invoke(NULL, image_capture_callback, (gpointer) all_data);
+							else
+                            	g_main_context_invoke(NULL, video_capture_callback, (gpointer) all_data);
+
                             break;
                     }
                     switch( event.key.keysym.sym )
@@ -495,11 +493,17 @@ void *main_loop(void *data)
                             g_print("q pressed - Quiting...\n");
                             break;
                         case SDLK_SPACE:
-                        {
+							{
                             if(global->AFcontrol > 0)
                                 setfocus_clicked(NULL, all_data);
-                        }
+							}
                             break;
+                        case SDLK_i:
+							g_main_context_invoke(NULL, image_capture_callback, (gpointer) all_data);
+							break;
+						case SDLK_v:
+							g_main_context_invoke(NULL, video_capture_callback, (gpointer) all_data);
+							break;
                         default:
                             break;
                     }
@@ -523,7 +527,7 @@ void *main_loop(void *data)
         }
         /* if set make the thread sleep - default no sleep (full throttle)*/
         if(global->vid_sleep) sleep_ms(global->vid_sleep);
-        
+
         /*------------------------------------------*/
         /*  restart video (new resolution/format)   */
         /*------------------------------------------*/
@@ -531,13 +535,13 @@ void *main_loop(void *data)
         {
             g_print("setting new resolution (%d x %d)\n", global->width, global->height);
             /*clean up */
-            
+
             if(particles) g_free(particles);
             particles = NULL;
-            
+
             if (global->debug) g_print("cleaning buffer allocations\n");
-            fflush(NULL);//flush all output buffers 
-            
+            fflush(NULL);//flush all output buffers
+
             if(!global->no_display)
             {
                 SDL_FreeYUVOverlay(overlay);
@@ -562,12 +566,12 @@ void *main_loop(void *data)
                 {
                     if (global->debug) g_print("yuv overlay created (%ix%i).\n", overlay->w, overlay->h);
                     p = (unsigned char *) overlay->pixels[0];
-        
+
                     drect.x = 0;
                     drect.y = 0;
                     drect.w = pscreen->w;
                     drect.h = pscreen->h;
-                    
+
                     global->change_res = FALSE;
                 }
             }
@@ -580,7 +584,7 @@ void *main_loop(void *data)
         capVid = videoIn->capVid;
     __UNLOCK_MUTEX(__VMUTEX);
     /*check if thread exited while in Video capture mode*/
-    if (capVid) 
+    if (capVid)
     {
         /*stop capture*/
         if (global->debug) g_print("stoping Video capture\n");
@@ -598,15 +602,15 @@ void *main_loop(void *data)
         __THREAD_JOIN( all_data->IO_thread );
         if (global->debug) g_print("IO Thread finished\n");
     }
-    
+
     if (global->debug) g_print("Thread terminated...\n");
     p = NULL;
     if(particles) g_free(particles);
     particles=NULL;
 
     if (global->debug) g_print("cleaning Thread allocations: 100%%\n");
-    fflush(NULL);//flush all output buffers 
-    
+    fflush(NULL);//flush all output buffers
+
     if(!global->no_display)
     {
         if(overlay)
@@ -615,9 +619,9 @@ void *main_loop(void *data)
 
         SDL_Quit();
     }
-	
+
     if (global->debug) g_print("Video thread completed\n");
-    
+
     global = NULL;
     AFdata = NULL;
     videoIn = NULL;
